package ca.recrown.islandsurvivalcraft.world.shaders;

import java.util.Random;

import org.bukkit.block.Biome;
import org.bukkit.util.noise.SimplexOctaveGenerator;

import ca.recrown.islandsurvivalcraft.world.IslandWorldBaseTerrainMap;

public class WorldHeightShader {
    private Random random;
    private SimplexOctaveGenerator noiseGenerator;
    private IslandWorldBaseTerrainMap islandLocator;
    private final float probabilityOfAdditive = 0.75f;

    public WorldHeightShader(long seed, IslandWorldBaseTerrainMap islandLocator) {
        random = new Random(seed);
        noiseGenerator = new SimplexOctaveGenerator(random, 2);
        noiseGenerator.setScale(0.05D);
        this.islandLocator = islandLocator;
    }

    public int getAltitude(int worldX, int worldZ, Biome biome) {
        double modifier = getHeightModifier(worldX, worldZ);
        int baseValue = islandLocator.getBaseTerrainHeight(worldX, worldZ);
        int height = 0;
        String biomeName = biome.name();
        if (biomeName.contains("hills")) {
            height = (int) (modifier * 40D + baseValue);
        } else if (biomeName.contains("mountains")) {
            height = (int) (modifier * 70D + baseValue);
        } else if (biomeName.contains("plateau")) {
            height = (int) (Math.min(60, Math.max(2, modifier * 80D)) + baseValue);
        } else if (biomeName.contains("modified")) {
            height = (int) (modifier * 50D + baseValue);
        } else if (biomeName.contains("shattered")) {
            height = (int) (modifier * 60D + baseValue);
        } else if (biomeName.contains("tall")) {
            height = (int) (modifier * 55D + baseValue);
        } else if (!biomeName.contains("ocean")) {
            height = (int) (getNormalizedHeightModifier(worldX, worldZ) * 20D + baseValue);
        } else {
            height = (int) (getNormalizedHeightModifier(worldX, worldZ) * 10D + baseValue);
        }
        return height;
    }

    private double getHeightModifier(int worldX, int worldZ) {
        return (2 * probabilityOfAdditive - noiseGenerator.noise(worldX, worldZ, 0.5D, 0.5D, true));
    }

    private double maxHeightModifier() {
        return 2 * probabilityOfAdditive + 1;
    }

    private double minHeightModifier() {
        return 2 * probabilityOfAdditive - 1;
    }

    private double getNormalizedHeightModifier(int worldX, int worldZ) {
        double heightModifier = getHeightModifier(worldX, worldZ);
        if (heightModifier > 0) {
            return heightModifier / maxHeightModifier();
        }
        return minHeightModifier();
    }
 }